@@ -82,6 +82,7 @@ sub get_download_info_as_json {
     push @cmd, '--dump-json', '--', $video_id;
     my $json;
     my $count = 1;
+
     RETRY: while ( 1 ) {
         my $not_ok;
         try {
@@ -91,7 +92,7 @@ sub get_download_info_as_json {
             print "\r", clline;
             print $message . "done.\n";
             print SHOW_CURSOR;
-            die if ! defined $json;
+            die if ! $json;
         }
         catch {
             say "$count/$opt->{retries}  $video_id: $_";
@@ -104,7 +105,9 @@ sub get_download_info_as_json {
             push @{$opt->{error_get_download_infos}}, $video_id;
             return;
         }
-        next RETRY if $not_ok;
+        if ( $not_ok ) {
+            next RETRY;
+        }
         last;
     }
     $opt->{up}++;
