@@ -22,34 +22,34 @@ use Try::Tiny        qw( try catch );
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
 use App::YTDL::Data   qw( get_new_video_url );
-use App::YTDL::Helper qw( sec_to_time insert_sep encode_fs encode_stdout );
-
-use constant {
-    HIDE_CURSOR => "\e[?25l",
-    SHOW_CURSOR => "\e[?25h",
-};
+use App::YTDL::Helper qw( sec_to_time insert_sep encode_fs encode_stdout HIDE_CURSOR SHOW_CURSOR );
 
 END { print SHOW_CURSOR }
 
 
+
 sub download_youtube {
     my ( $opt, $info ) = @_;
     my $ua = LWP::UserAgent->new( agent => $opt->{useragent}, show_progress => 0 );
+    my $nr = 1;
     for my $video_id ( sort { $info->{$a}{count} <=> $info->{$b}{count} } keys %$info ) {
-        try {
-            my $file_name_OS = encode_fs( $info->{$video_id}{file_name} );
-            unlink $file_name_OS or die $! if -f $file_name_OS && $opt->{overwrite};
-            _download_video( $opt, $info, $ua, $video_id );
-            if ( $opt->{upload_date_is_file_date} ) {
-                if ( $info->{$video_id}{upload_date} =~ /^(\d\d\d\d)-(\d\d)-(\d\d)\z/ ) {
-                    my $time = timelocal( 0, 0, 0, $3, $2 - 1, $1 );
-                    my $ref = File::Touch->new( time => $time );
-                    my $count = $ref->touch( $file_name_OS );
+        for my $fmt ( sort keys %{$info->{$video_id}{video_url}} ) {
+            try {
+                my $file_name_OS = encode_fs( $info->{$video_id}{file_name}{$fmt} );
+                unlink $file_name_OS or die $! if -f $file_name_OS && $opt->{overwrite};
+                _download_video( $opt, $info, $ua, $video_id, $fmt, $nr );
+                if ( $opt->{upload_date_is_file_date} ) {
+                    if ( $info->{$video_id}{upload_date} =~ /^(\d\d\d\d)-(\d\d)-(\d\d)\z/ ) {
+                        my $time = timelocal( 0, 0, 0, $3, $2 - 1, $1 );
+                        my $ref = File::Touch->new( time => $time );
+                        my $count = $ref->touch( $file_name_OS );
+                    }
                 }
             }
-        }
-        catch {
-            say "$video_id - $_";
+            catch {
+                say "$video_id - $_";
+            };
+            $nr++;
         }
     }
     return;
@@ -57,10 +57,9 @@ sub download_youtube {
 
 
 sub _download_video {
-    my ( $opt, $info, $ua, $video_id ) = @_;
-    my $nr            = $info->{$video_id}{count};
-    my $video_url     = $info->{$video_id}{video_url};
-    my $file_name     = $info->{$video_id}{file_name};
+    my ( $opt, $info, $ua, $video_id, $fmt, $nr ) = @_;
+    my $video_url     = $info->{$video_id}{video_url}{$fmt};
+    my $file_name     = $info->{$video_id}{file_name}{$fmt};
     my $file_name_OS  = encode_fs( $file_name );
     my $file_basename = basename $file_name;
     print HIDE_CURSOR;
@@ -79,17 +78,19 @@ sub _download_video {
     RETRY: while ( 1 ) {
         $p->{size}      = -s $file_name_OS // 0;
         $p->{starttime} = gettimeofday;
-        my $retries = ' ' x ( length( $opt->{retries} ) * 2 + 1 );
+        my $retries = sprintf "%*s/%s", length $opt->{retries}, $try, $opt->{retries};
         if ( $try > 1 ) {
-            $retries = sprintf "%*s/%s", length $opt->{retries}, $try, $opt->{retries};
-            $video_count = ' ' x length( $video_count );
+            $video_count = ' ' x length $video_count;
+        }
+        else {
+            $retries = ' ' x length $retries;
         }
         my $res;
         if ( ! $p->{size} ) {
             my $at = '';
             open my $fh, '>:raw', $file_name_OS or die $!;
             printf _p_fmt( $opt, "start" ), $video_count, $retries, $at;
-            _log_info( $opt, $info, $video_id ) if $opt->{log_info};
+            _log_info( $opt, $info, $video_id, $fmt ) if $opt->{log_info};
             $res = $ua->get(
                 $video_url,
                 ':content_cb' => _return_callback( $opt, $fh, $p ),
@@ -113,46 +114,51 @@ sub _download_video {
         my $dl_time = sec_to_time( int( tv_interval( [ $p->{starttime} ] ) ), 1 );
         my $size = '';
         my $avg_speed = '';
-        my $at   = '';
+        my $at = '';
         my $pr_status = 'status ' . $status;
         my $incomplete = '';
         if ( $status =~ /^(200|206|416)/ ) {
             my $file_size = -s $file_name_OS // -1;
             $avg_speed = sprintf "avg %2sk/s", $p->{kbs_avg} || '--';
             if ( $p->{total} ) {
+                my $len_total_mb = length( int( $p->{total} / 1024 ** 2 ) ) + 2;
+                $size = sprintf "%*.2fM",  $len_total_mb, $file_size / 1024 ** 2;
+                $at   = sprintf "@%*.2fM", $len_total_mb, $p->{size} / 1024 ** 2;
                 if ( $file_size != $p->{total} ) {
-                    my $pr_total = insert_sep( $p->{total} );
-                    $incomplete = sprintf " Incomplete: %*s/%s ", length( $pr_total ), insert_sep( $file_size ), $pr_total;
+                    my $should = sprintf "%.2fM", $p->{total} / 1024 ** 2;
+                    my $is     = sprintf "%.2fM", $file_size  / 1024 ** 2;
+                    if ( $should eq $is ) {
+                        $should = insert_sep( $p->{total} );
+                        $is     = insert_sep( $file_size );
+                    }
+                    $incomplete = sprintf " Incomplete: %*s/%s ", length( $should ), $is, $should;
                     $retries    = sprintf "%*s/%s", length $opt->{retries}, $try, $opt->{retries};
                 }
-                my $length_total_mb = length( int( $p->{total} / 1024 ** 2 ) ) + 2;
-                $size = sprintf "%*.2fM", $length_total_mb, $file_size / 1024 ** 2;
-                $at   = sprintf "@%*.2fM", $length_total_mb, $p->{size} / 1024 ** 2;
-                if ( $status == 416 ) {
-                    $dl_time   = '';
-                    $size      = ' ' x ( $length_total_mb + 2 );
-                    $avg_speed = '';
-                }
-                if ( $status == 206 ) {
-                }
-                if ( $status =~ 200 ) {
-                    $at        = ' ' x ( $length_total_mb + 3 );
-                    $pr_status = ' ' x 10;
-                }
             }
             if ( $status == 416 ) {
+                $dl_time   = ' ' x length '0:00:00';
+                $size      = ' ' x length $size;
+                $avg_speed = ' ' x length $avg_speed;
                 printf _p_fmt( $opt, "status_416" ), $video_count, $retries, $dl_time, $at, $pr_status;
+                last RETRY;
             }
             else {
+                if ( $status == 200 ) {
+                    $at        = ' ' x length $at;
+                    $pr_status = ' ' x length $pr_status;
+                }
                 printf _p_fmt( $opt, "status" ), $video_count, $retries, $dl_time, $size, $avg_speed, $at, $pr_status, $incomplete;
+                last RETRY if $p->{total} && $p->{total} == $file_size;
             }
-            last RETRY if $p->{total} && $p->{total} == $file_size;
-            last RETRY if $status == 416;
         }
         else {
-            $retries = sprintf "%*s/%s", length $opt->{retries}, $try, $opt->{retries};
-            $pr_status = 'status ' . $status . '  Fetching new video url ...';
-            my $new_video_url = get_new_video_url( $opt, $info, $video_id );
+            $retries    = sprintf "%*s/%s", length $opt->{retries}, $try, $opt->{retries};
+            $size       = ' ';
+            $avg_speed  = ' ';
+            $at         = ' ';
+            $pr_status  = 'status ' . $status . '  Fetching new video url ...';
+            $incomplete = ' ';
+            my $new_video_url = get_new_video_url( $opt, $info, $video_id, $fmt );
             if ( ! $new_video_url ) {
                 $pr_status .=  ' failed!';
             }
@@ -166,7 +172,7 @@ sub _download_video {
         }
         $try++;
         if ( $try > $opt->{retries} ) {
-            push @{$opt->{incomplete_download}}, $video_url;
+            push @{$opt->{incomplete_download}}, "$video_id : $file_name";
             last RETRY;
         }
         sleep 4 * $try;
@@ -177,7 +183,7 @@ sub _download_video {
 
 
 sub _log_info {
-    my ( $opt, $info, $video_id ) = @_;
+    my ( $opt, $info, $video_id, $fmt ) = @_;
     my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime;
     my $id = $info->{$video_id}{extractor_key} eq 'Youtube' ? $video_id : $info->{$video_id}{extractor_key} . ' ' . $video_id;
     my $log_str = sprintf( "%04d-%02d-%02d %02d:%02d", $year + 1900, $mon + 1, $mday, $hour, $min )
@@ -185,7 +191,7 @@ sub _log_info {
         . ' | ' . ( $info->{$video_id}{uploader_id} // 'xxxxx' )
         . ' | ' . ( $info->{$video_id}{playlist_id} // '-----' )
         . ' | ' . ( $info->{$video_id}{upload_date} )
-        . '   ' . basename( $info->{$video_id}{file_name} );
+        . '   ' . basename( $info->{$video_id}{file_name}{$fmt} );
     open my $log, '>>:encoding(UTF-8)', encode_fs( $opt->{log_file} ) or die $!;
     flock $log, LOCK_EX     or die $!;
     seek  $log, 0, SEEK_END or die $!;
@@ -198,9 +204,9 @@ sub _p_fmt {
     my ( $opt, $key ) = @_;
     my $sl = $opt->{kb_sec_len} + 7;
     my %hash = (
-        start        => "  %s  %3s  %9s\n", ###
-        status_416   => "  %s  %3s  %7s  %9s   %s\n", #
-        status       => "  %s  %3s  %7s  %9s  %${sl}s   %s    %s  %s\n",
+        start        => "  %s  %s  %9s\n",
+        status_416   => "  %s  %s  %s  %9s   %s\n",
+        status       => "  %s  %s  %s  %9s  %${sl}s   %s    %s  %s\n",
         info_row1    => "%9.*f %s %37s %"    . (      $opt->{kb_sec_len} ) . "sk/s\n",
         info_row2    => "%9.*f %s %6.*f%% %" . ( 30 + $opt->{kb_sec_len} ) . "sk/s\n",
         info_nt_row1 => " %34s %24sk/s\n",
