@@ -7,7 +7,8 @@ use 5.010000;
 
 use Exporter qw( import );
 our @EXPORT_OK = qw( term_size sec_to_time insert_sep unicode_trim choose_a_dir choose_a_number
-                     print_hash encode_fs encode_stdout_lax encode_stdout write_json read_json );
+                     print_hash encode_fs encode_stdout_lax encode_stdout write_json read_json
+                     HIDE_CURSOR SHOW_CURSOR );
 
 use Encode             qw( encode );
 use Unicode::Normalize qw( NFC );
@@ -17,6 +18,12 @@ use JSON               qw();
 use Term::Choose::Util qw( print_hash choose_a_dir choose_a_number term_size insert_sep unicode_trim );
 
 
+use constant {
+    HIDE_CURSOR => "\e[?25l",
+    SHOW_CURSOR => "\e[?25h",
+};
+
+
 sub encode_fs {
     my ( $filename ) = @_;
     return encode( 'locale_fs', NFC( $filename ), Encode::FB_HTMLCREF );
